/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>

#include "StereoElecController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_elec {

const string StereoElecController::TYPE = "stereo_elec_controller";

/*
 * Constructor
 */
StereoElecController::StereoElecController(const string& name) :ExperimentController(name){


	setFamily(family::SAMPLE_ENVIRONMENT);

	// Init property
	nbfemax.init(this, NOSAVE, "nbfemax");
	nbledmax.init(this, NOSAVE, "nbledmax");
	slotindex.init(this, NOSAVE, "slotindex");
	fe_pos5v.init(this, NOSAVE, "fe_pos5v");
	fe_neg5v.init(this, NOSAVE, "fe_neg5v");
	fe_adc3v3a.init(this, NOSAVE, "fe_adc3v3a");
	fe_adc1v8a.init(this, NOSAVE, "fe_adc1v8a");
	fe_tadc03.init(this, NOSAVE, "fe_tadc03");
	fe_tadc47.init(this, NOSAVE, "fe_tadc47");
	led_dac3v3.init(this, NOSAVE, "led_dac3v3");
	led_amp24v.init(this, NOSAVE, "led_amp24v");

	rate0.init(this, NOSAVE, "rate0");
	rate1.init(this, NOSAVE, "rate1");
	rate2.init(this, NOSAVE, "rate2");
	rate3.init(this, NOSAVE, "rate3");
	rate4.init(this, NOSAVE, "rate4");
	rate5.init(this, NOSAVE, "rate5");
	rate6.init(this, NOSAVE, "rate6");
	rate7.init(this, NOSAVE, "rate7");

	m_Driver.init(this, "driver");

	registerFunction(TYPE);
	registerRefresher(nbfemax,&StereoElecController::refreshNbFEMaxProperty, this);
	registerRefresher(nbledmax,&StereoElecController::refreshNbLEDMaxProperty, this);
	nbfemax=NBFEMAX;
	nbledmax=NBLEDMAX;

}

StereoElecController::StereoElecController(const StereoElecController& controller) :
	ExperimentController(controller){
	nbfemax.copy(this, controller.nbfemax);
	nbledmax.copy(this, controller.nbledmax);
	slotindex.copy(this, controller.slotindex);
	fe_pos5v.copy(this, controller.fe_pos5v);
	fe_neg5v.copy(this, controller.fe_neg5v);
	fe_adc3v3a.copy(this, controller.fe_adc3v3a);
	fe_adc1v8a.copy(this, controller.fe_adc1v8a);
	fe_tadc03.copy(this, controller.fe_tadc03);
	fe_tadc47.copy(this, controller.fe_tadc47);
	led_dac3v3.copy(this, controller.led_dac3v3);
	led_amp24v.copy(this, controller.led_amp24v);

	rate0.copy(this, controller.rate0);
	rate1.copy(this, controller.rate1);
	rate2.copy(this, controller.rate2);
	rate3.copy(this, controller.rate3);
	rate4.copy(this, controller.rate4);
	rate5.copy(this, controller.rate5);
	rate6.copy(this, controller.rate6);
	rate7.copy(this, controller.rate7);

	m_Driver.copy(controller.m_Driver);

}

/*
 * Destructor
 */
StereoElecController::~StereoElecController() {
	// Empty
}


void StereoElecController::postConfiguration() {

	registerUpdater(m_Driver->fe_pos5v, &StereoElecController::updateFE_Pos5V, this);
	registerUpdater(m_Driver->fe_neg5v, &StereoElecController::updateFE_Neg5V, this);
	registerUpdater(m_Driver->fe_adc3v3a, &StereoElecController::updateFE_ADC3V3A, this);
	registerUpdater(m_Driver->fe_adc1v8a, &StereoElecController::updateFE_ADC1V8A, this);
	registerUpdater(m_Driver->fe_tadc03, &StereoElecController::updateFE_TADC03, this);
	registerUpdater(m_Driver->fe_tadc47, &StereoElecController::updateFE_TADC47, this);
	registerUpdater(m_Driver->led_dac3v3, &StereoElecController::updateLED_DAC3V3, this);
	registerUpdater(m_Driver->led_amp24v, &StereoElecController::updateLED_Amp24V, this);

	registerUpdater(m_Driver->rate0, &StereoElecController::updateRate0, this);
	registerUpdater(m_Driver->rate1, &StereoElecController::updateRate1, this);
	registerUpdater(m_Driver->rate2, &StereoElecController::updateRate2, this);
	registerUpdater(m_Driver->rate3, &StereoElecController::updateRate3, this);
	registerUpdater(m_Driver->rate4, &StereoElecController::updateRate4, this);
	registerUpdater(m_Driver->rate5, &StereoElecController::updateRate5, this);
	registerUpdater(m_Driver->rate6, &StereoElecController::updateRate6, this);
	registerUpdater(m_Driver->rate7, &StereoElecController::updateRate7, this);

	m_Driver->execute("readMonitoring");

}
void StereoElecController::refreshNbFEMaxProperty(int32 value) throw (CannotSetValue){
	slotindex.resize(value);
	for(int i=0;i<value;i++) slotindex.set(i,i);
	fe_pos5v.resize(value);
	fe_neg5v.resize(value);
	fe_adc3v3a.resize(value);
	fe_adc1v8a.resize(value);
	fe_tadc03.resize(value);
	fe_tadc47.resize(value);

	rate0.resize(value);
	rate1.resize(value);
	rate2.resize(value);
	rate3.resize(value);
	rate4.resize(value);
	rate5.resize(value);
	rate6.resize(value);
	rate7.resize(value);

}

void StereoElecController::refreshNbLEDMaxProperty(int32 value) throw (CannotSetValue){
	led_dac3v3.resize(value);
	led_amp24v.resize(value);

}

void StereoElecController::updateFE_Pos5V(int32 index){
	fe_pos5v.set(index,m_Driver->fe_pos5v(index));
}
void StereoElecController::updateFE_Neg5V(int32 index){
	fe_neg5v.set(index,m_Driver->fe_neg5v(index));
}
void StereoElecController::updateFE_ADC3V3A(int32 index){
	fe_adc3v3a.set(index,m_Driver->fe_adc3v3a(index));
}
void StereoElecController::updateFE_ADC1V8A(int32 index){
	fe_adc1v8a.set(index,m_Driver->fe_adc1v8a(index));
}
void StereoElecController::updateFE_TADC03(int32 index){
	fe_tadc03.set(index,m_Driver->fe_tadc03(index));
}
void StereoElecController::updateFE_TADC47(int32 index){
	fe_tadc47.set(index,m_Driver->fe_tadc47(index));
}
void StereoElecController::updateLED_DAC3V3(int32 index){
	led_dac3v3.set(index,m_Driver->led_dac3v3(index));
}
void StereoElecController::updateLED_Amp24V(int32 index){
	led_amp24v.set(index,m_Driver->led_amp24v(index));
}

void StereoElecController::updateRate0(int32 index){
	rate0.set(index,m_Driver->rate0(index));
}
void StereoElecController::updateRate1(int32 index){
	rate1.set(index,m_Driver->rate1(index));
}
void StereoElecController::updateRate2(int32 index){
	rate2.set(index,m_Driver->rate2(index));
}
void StereoElecController::updateRate3(int32 index){
	rate3.set(index,m_Driver->rate3(index));
}
void StereoElecController::updateRate4(int32 index){
	rate4.set(index,m_Driver->rate4(index));
}
void StereoElecController::updateRate5(int32 index){
	rate5.set(index,m_Driver->rate5(index));
}
void StereoElecController::updateRate6(int32 index){
	rate6.set(index,m_Driver->rate6(index));
}
void StereoElecController::updateRate7(int32 index){
	rate7.set(index,m_Driver->rate7(index));
}

}

